/*
 * $RCSfile: NetworkInfo.cc,v $
 * $Revision: 1.1 $
 *
 * Comments:
 *
 * (C) Copyright ParaSoft Corporation 1996.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 *
 * $Author: mlyko $          $Locker:  $
 * $Date: 2005-08-23 09:09:14 $
 * $Log: not supported by cvs2svn $
 */

#include <stdio.h>
#include <unistd.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>

#include "NetworkInfo.h"

char ip_address[20];
char net_mask[20];
char broadcast[20];

NetworkInfo::NetworkInfo()
{
}

NetworkInfo::~NetworkInfo()
{
}

void NetworkInfo::ReadNetworkInfo(char* ip_address, char* net_mask, char* broadcast)
{
    int tmpSock;

    printf("ReadNetworkInfo() started... \n");

    if ((tmpSock = socket(AF_INET,  SOCK_DGRAM, 0)) < 0) {
        perror ("socket creation");
        return;
    }

    struct sockaddr_in  netmask,bcast;
    struct ifreq *ifrP;
    struct ifconf ifc;
    char buf[4096];
    int n, found=0;

    /* loop among all interfaces */
    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (ioctl(tmpSock, SIOCGIFCONF, (caddr_t)&ifc) < 0) {
        perror("ioctl (SIOCGIFCONF)");
        return;
    }

    ifrP = ifc.ifc_req;
    for (n=ifc.ifc_len/sizeof(struct ifreq); --n>=0; ifrP++) {
        if (ifrP->ifr_addr.sa_family != AF_INET)
            continue;           /* not internet domain */
        if (ioctl(tmpSock, SIOCGIFFLAGS, (caddr_t)ifrP) < 0) {
            perror("ioctl (SIOCGIFFLAGS)");
            return;
        }
    /* skip uninteresting cases */
        if ((ifrP->ifr_flags & IFF_UP) == 0 || (ifrP->ifr_flags & IFF_LOOPBACK) ){
            continue;
        }
        found = 1;
        break;
    }
  
    /* ok. get IP number and netmask for this interface (hope this is the good 
       one. It may be wrong in the case there are several ones...) */
    if (!found) return;

    if (ioctl(tmpSock, SIOCGIFADDR, (caddr_t)ifrP) < 0) {
        if (errno != EADDRNOTAVAIL)
            perror("ioctl (SIOCGIFADDR)");
        return ;
    }

    strcpy(ip_address, inet_ntoa((((struct sockaddr_in *)&ifrP->ifr_addr)->sin_addr)));

    if (ioctl(tmpSock, SIOCGIFBRDADDR, (caddr_t)ifrP) < 0) {
        if (errno != EADDRNOTAVAIL)
            perror("ioctl (SIOCGIFBRDADDR)");
        return;
    }
    bcast.sin_addr =  ((struct sockaddr_in *)&ifrP->ifr_addr)->sin_addr;
    strcpy(broadcast, inet_ntoa(bcast.sin_addr));
    
    if (ioctl(tmpSock, SIOCGIFNETMASK, (caddr_t)ifrP) < 0) {
        if (errno != EADDRNOTAVAIL)
            perror("ioctl (SIOCGIFNETMASK)");
        return;
    }
    netmask.sin_addr =  ((struct sockaddr_in *)&ifrP->ifr_addr)->sin_addr;

    strcpy(net_mask, inet_ntoa((((struct sockaddr_in *)&ifrP->ifr_addr)->sin_addr)));
    
    close (tmpSock);
}

